package com.tofly.yxpc.utils.db;

import android.util.Log;

import com.tofly.yxpc.base.MyApp;
import com.tofly.yxpc.greenDao.DaoSession;

import org.greenrobot.greendao.AbstractDao;
import org.greenrobot.greendao.Property;
import org.greenrobot.greendao.query.QueryBuilder;
import org.greenrobot.greendao.query.WhereCondition;

import java.util.ArrayList;
import java.util.List;

/**
 * 数据库基础封装
 */
public class DbUtils<T> {
    private static final String TAG = DbUtils.class.getSimpleName();

    private DaoSession daoSession;
    private Class<T> entityClass;
    private AbstractDao<T, Long> entityDao;

    public DbUtils(Class<T> pEntityClass, AbstractDao<T, Long> pEntityDao) {
        daoSession = MyApp.getInstance().getDaoSession();
        entityClass = pEntityClass;
        entityDao = pEntityDao;
    }

    /**
     * 插入记录，如果表未创建，先创建表
     *
     * @param pEntity
     * @return
     */
    public boolean insert(T pEntity) {
        boolean flag = entityDao.insert(pEntity) == -1 ? false : true;
        Log.i(TAG, "insert Meizi :" + flag + "-->" + pEntity.toString());
        return flag;
    }


    /**
     * 插入记录，如果表未创建，先创建表
     *
     * @param pEntity
     * @return
     */
    public boolean insertOrReplace(T pEntity) {
        boolean flag = entityDao.insertOrReplace(pEntity) == -1 ? false : true;
        Log.i(TAG, "insert Meizi :" + flag + "-->" + pEntity.toString());
        return flag;
    }

    /**
     * 插入多条数据，在子线程操作
     *
     * @param pEntityList
     * @return
     */
    public boolean insertMulti(final List<T> pEntityList, boolean delete) {
        try {
            if (delete) {
                entityDao.detachAll();
                entityDao.deleteAll();
            }
            daoSession.runInTx(() -> {
                for (T meizi : pEntityList) {
                    daoSession.insertOrReplace(meizi);
                }
            });
            return true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }


    /**
     * 修改一条数据
     *
     * @param pEntity
     * @return
     */
    public boolean update(T pEntity) {
        try {
            daoSession.update(pEntity);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * 删除单条记录
     *
     * @param pEntity
     * @return
     */
    public boolean delete(T pEntity) {
        try {
            //按照id删除
            daoSession.delete(pEntity);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }
    /**
     * 批量删除
     * @param entityList
     * @return
     */
    public boolean delete(List<T> entityList) {
        try {
            for (T pEntity:entityList) {
                //按照id删除
                daoSession.delete(pEntity);
            }
            return true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }
    /**
     * 删除所有记录
     *
     * @return
     */
    public boolean deleteAll() {
        try {
            //按照id删除
            daoSession.deleteAll(entityClass);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * 查询所有记录
     *
     * @return
     */
    public List<T> queryAll() {
        return daoSession.loadAll(entityClass);
    }

    /**
     * 根据主键id查询记录
     *
     * @param key
     * @return
     */
    public T queryById(long key) {
        return daoSession.load(entityClass, key);
    }

    /**
     * 使用native sql进行查询操作
     */
    public List<T> queryByNativeSql(String sql, String[] conditions) {
        return daoSession.queryRaw(entityClass, sql, conditions);
    }


    /**
     * 使用queryBuilder进行查询
     *
     * @return
     */
    public List<T> queryByQueryBuilder(WhereCondition cond) {
        QueryBuilder<T> queryBuilder = daoSession.queryBuilder(entityClass);
        return queryBuilder.where(cond).list();
    }
    /**
     * 使用queryBuilder进行查询
     *
     * @return
     */
    public long queryCountByQueryBuilder() {
        QueryBuilder<T> queryBuilder = daoSession.queryBuilder(entityClass);
        return queryBuilder.count();
    }
    /**
     * 使用queryBuilder进行查询
     *
     * @return
     */
    public long queryCountByQueryBuilder(WhereCondition cond) {
        QueryBuilder<T> queryBuilder = daoSession.queryBuilder(entityClass);
        return queryBuilder.where(cond).count();
    }
    public long queryCountByQueryBuilder(WhereCondition cond, WhereCondition... condMore) {
        QueryBuilder<T> queryBuilder = daoSession.queryBuilder(entityClass);
        return queryBuilder.where(cond, condMore).count();
    }
    /**
     * 使用queryBuilder进行查询
     *
     * @return
     */
    public T queryByQueryBuilderOne(WhereCondition cond) {
        QueryBuilder<T> queryBuilder = daoSession.queryBuilder(entityClass);
        return queryBuilder.where(cond).unique();
    }

    /**
     * 使用queryBuilder进行查询
     *
     * @return
     */
    public List<T> queryByQueryBuilder(WhereCondition cond, WhereCondition... condMore) {
        QueryBuilder<T> queryBuilder = daoSession.queryBuilder(entityClass);
        return queryBuilder.where(cond, condMore).list();
    }

    /**
     * 使用queryBuilder进行查询
     *
     * @return 按照 orderBy 排序 升序
     */
    public List<T> queryByQueryBuilderByOrderAsc(Property orderBy, WhereCondition cond, WhereCondition... condMore) {
        QueryBuilder<T> queryBuilder = daoSession.queryBuilder(entityClass);
        return queryBuilder.where(cond, condMore).orderAsc(orderBy).list();
    }

    /**
     * 使用queryBuilder或者进行查询
     * 条件分成了2类 0-n 属于and条件  其他为or条件
     * <p>
     * <p>
     * 如果还有类似的需求查询  两个地方的都用数组优化
     *
     * @return
     */
    public List<T> queryByQueryBuilderOrContainAnd(WhereCondition cond00, WhereCondition cond01, WhereCondition cond02, WhereCondition cond03, WhereCondition cond, WhereCondition cond1, WhereCondition... condMore) {
        QueryBuilder<T> queryBuilder = daoSession.queryBuilder(entityClass);

        List<WhereCondition> addCondition = new ArrayList<WhereCondition>();
        if (cond00 != null) {
            addCondition.add(cond00);
        }
        if (cond01 != null) {
            addCondition.add(cond01);
        }
        if (cond02 != null) {
            addCondition.add(cond02);
        }
        if (cond03 != null) {
            addCondition.add(cond03);
        }
        WhereCondition[] edges;
        int size = addCondition.size();
        if (size > 0) {
            edges = new WhereCondition[size];
            for (int i = 0; i < size; i++) {
                edges[i] = addCondition.get(i);
            }

            return queryBuilder.where(edges[0], edges).whereOr(cond, cond1, condMore).list();
        } else {
            return queryBuilder.whereOr(cond, cond1, condMore).list();
        }
    }

    public List<T> queryByQueryBuilderOrContainAnd2(WhereCondition cond00, WhereCondition cond01, WhereCondition cond02, WhereCondition cond03) {
        QueryBuilder<T> queryBuilder = daoSession.queryBuilder(entityClass);

        List<WhereCondition> addCondition = new ArrayList<WhereCondition>();
        if (cond00 != null) {
            addCondition.add(cond00);
        }
        if (cond01 != null) {
            addCondition.add(cond01);
        }
        if (cond02 != null) {
            addCondition.add(cond02);
        }
        if (cond03 != null) {
            addCondition.add(cond03);
        }
        WhereCondition[] edges;
        int size = addCondition.size();
        if (size > 0) {
            edges = new WhereCondition[size];
            for (int i = 0; i < size; i++) {
                edges[i] = addCondition.get(i);
            }

            return queryBuilder.where(edges[0], edges).list();
        } else {
            return null;
        }
    }

    /**
     * 使用queryBuilder或者进行查询
     *
     * @return
     */
    public List<T> queryByQueryBuilderOr(WhereCondition cond, WhereCondition cond1, WhereCondition... condMore) {
        QueryBuilder<T> queryBuilder = daoSession.queryBuilder(entityClass);
        return queryBuilder.whereOr(cond, cond1, condMore).list();
    }


    /**
     * @param cond0    相等
     * @param cond     模糊查询 or
     * @param cond1    模糊查询 or
     * @param condMore 模糊查询 or
     * @return
     */
    public List<T> queryByQueryBuilderAndOr(WhereCondition cond0, WhereCondition cond, WhereCondition cond1, WhereCondition... condMore) {
        QueryBuilder<T> queryBuilder = daoSession.queryBuilder(entityClass);
        return queryBuilder.where(cond0).whereOr(cond, cond1, condMore).list();
    }


    /**
     * 使用queryBuilder进行查询
     *
     * @return
     */
    public T queryByQueryBuilderOne(WhereCondition cond, WhereCondition... condMore) {
        QueryBuilder<T> queryBuilder = daoSession.queryBuilder(entityClass);
        return queryBuilder.where(cond, condMore).unique();
    }
}
