package com.tofly.latte_core.base;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.graphics.Point;
import android.os.Build;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.view.View;
import android.view.ViewConfiguration;
import android.view.WindowManager;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.ActionBar;
import androidx.appcompat.widget.Toolbar;

import com.tofly.latte_core.utils.ActivityUtils;
import com.tofly.latte_core.utils.ToastUtils;
import com.trello.rxlifecycle2.components.support.RxAppCompatActivity;

import butterknife.ButterKnife;
import qiu.niorgai.StatusBarCompat;

/**
 * @author ychk
 */
public abstract class BaseActivity<P extends IPresenter> extends RxAppCompatActivity implements BaseView {

    public P mPresenter;
    private ProgressDialog dialog;
    private static final int PERMISSION_CODE = 1000;
    private OnPermissionListener listener = null;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_ADJUST_PAN);
        Object object = setLayout();
        if (object instanceof Integer) {
            Integer resId = (Integer) object;
            setContentView(resId);
        } else if (object instanceof View) {
            View rootView = (View) setLayout();
            setContentView(rootView);
        }
        StatusBarCompat.translucentStatusBar(this, true);
        ActivityUtils.getInstance().addActivity(this);
        ButterKnife.bind(this);
        dialog = new ProgressDialog(this);
        dialog.setCancelable(false);
        dialog.setCanceledOnTouchOutside(false);
        mPresenter = createPresenter();
        if (mPresenter != null) {
            mPresenter.onAttachView(this);
        }
        initView();
    }

    /**
     * 接入视图
     *
     * @return 视图id
     */
    public abstract Object setLayout();

    /**
     * 创建presenter类
     *
     * @return 管理类
     */
    protected abstract P createPresenter();

    /**
     * 初始加载视图
     */
    public abstract void initView();

    /**
     * 初始化导航栏
     */
    public void initToolBar(Toolbar toolbar, TextView title, String titleName) {
        if (toolbar != null) {
            setSupportActionBar(toolbar);
            ActionBar actionBar = getSupportActionBar();
            if (actionBar != null) {
                actionBar.setDisplayHomeAsUpEnabled(true);
                actionBar.setDisplayShowTitleEnabled(false);
            }
            title.setText(titleName == null ? "" : titleName);
            toolbar.setNavigationOnClickListener(v -> {
                if (backAfter()) {
                    this.finish();
                }
            });
        }
    }

    /**
     * 拦截返回操作
     */
    public boolean backAfter() {
        return true;
    }

    /**
     * 检查权限
     */
    public void checkPermission(OnPermissionListener listener, @NonNull String... permissions) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (permissions.length > 0) {
                this.listener = listener;
                boolean isSuccess = true;
                for (String s : permissions) {
                    if (checkSelfPermission(s) != PackageManager.PERMISSION_GRANTED) {
                        requestPermissions(permissions, PERMISSION_CODE);
                        isSuccess = false;
                    }
                }
                if (isSuccess) {
                    listener.onPermissionResult(true);
                }
            }
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == PERMISSION_CODE) {
            if (grantResults.length > 0) {
                boolean isSuccess = true;
                for (int i : grantResults) {
                    if (i != PackageManager.PERMISSION_GRANTED) {
                        isSuccess = false;
                        break;
                    }
                }
                listener.onPermissionResult(isSuccess);
            }
        }
    }

    @Override
    public void onMessage(String message) {
        ToastUtils.showToast(message);
    }

    @Override
    public void showLoading(String msg) {
        if (dialog != null) {
            dialog.setMessage(msg);
            if (!dialog.isShowing()) {
                dialog.show();
            }
        }
    }

    @Override
    public void hideLoading() {
        if (dialog != null) {
            if (dialog.isShowing()) {
                dialog.dismiss();
            }
        }
    }

    @Override
    protected void onStop() {
        super.onStop();
        if (dialog != null) {
            dialog.dismiss();
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (mPresenter != null) {
            mPresenter.onDetachView();
        }
        ActivityUtils.getInstance().finishActivity(this);
    }

    /**
     * 算取状态栏的高
     */
    public static int getStatusBarHeight(Context context) {
        int result = 0;
        int resourceId = context.getResources().getIdentifier("status_bar_height", "dimen", "android");
        if (resourceId > 0) {
            result = context.getResources().getDimensionPixelSize(resourceId);
        }
        return result;
    }

    /**
     * 算取状态栏的高
     */
    public static int getNavigationBarHeight(Context var0) {
        boolean var1 = ViewConfiguration.get(var0).hasPermanentMenuKey();
        int var2;
        return (var2 = var0.getResources().getIdentifier("navigation_bar_height", "dimen", "android")) > 0 && !var1 ? var0.getResources().getDimensionPixelSize(var2) : 0;
    }

    /**
     * 获取当前视图高度
     */
    public static int getViewHeight(Context context) {
        return getScreenHeight(context) - getStatusBarHeight(context) - getNavigationBarHeight(context);
    }

    /**
     * 屏幕高度
     */
    public static int getScreenHeight(Context context) {
        WindowManager wm = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
        Point point = new Point();
        wm.getDefaultDisplay().getRealSize(point);
        return point.y;
    }

    /**
     * 设置字体大小
     */
    public void setTypeFace(float size) {
        Configuration configuration = getResources().getConfiguration();
        configuration.fontScale = size;
        DisplayMetrics metrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(metrics);
        metrics.scaledDensity = configuration.fontScale * metrics.density;
        getResources().updateConfiguration(configuration, metrics);
    }

    public interface OnPermissionListener {
        void onPermissionResult(boolean isSuccess);
    }

}
