package com.tofly.latte_core.base;

import android.app.ProgressDialog;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.ActionBar;
import androidx.appcompat.widget.Toolbar;

import com.tofly.latte_core.utils.ToastUtils;
import com.trello.rxlifecycle2.components.support.RxAppCompatActivity;
import com.trello.rxlifecycle2.components.support.RxFragment;

import butterknife.ButterKnife;
import butterknife.Unbinder;

/**
 * @author ychk
 */
public abstract class BaseDelegate<P extends IPresenter> extends RxFragment implements BaseView {

    private ProgressDialog dialog;
    private Unbinder unbinder;
    public View rootView = null;
    protected P mPresenter;
    private static final int PERMISSION_CODE = 1000;
    private OnPermissionListener listener = null;
    protected Bundle savedInstanceState = null;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        this.savedInstanceState = savedInstanceState;
        if (setLayout() instanceof Integer) {
            rootView = inflater.inflate((Integer) setLayout(), container, false);
        } else if (setLayout() instanceof View) {
            rootView = (View) setLayout();
        }
        unbinder = ButterKnife.bind(this, rootView);
        dialog = new ProgressDialog(getContext());
        dialog.setCancelable(false);
        mPresenter = createPresenter();
        if (mPresenter != null) {
            mPresenter.onAttachView(this);
        }
        setHasOptionsMenu(true);
        initView();
        return rootView;
    }

    /**
     * 创建布局对象
     *
     * @return
     */
    public abstract Object setLayout();

    /**
     * 创建presenter对象
     *
     * @return
     */
    protected abstract P createPresenter();

    /**
     * 初始化数据
     */
    public abstract void initView();

    @Override
    public void onMessage(String message) {
        ToastUtils.showToast(message);
    }

    @Override
    public void showLoading(String msg) {
        if (dialog != null) {
            dialog.setMessage(msg);
            if (!dialog.isShowing()) {
                dialog.show();
            }
        }
    }

    @Override
    public void hideLoading() {
        if (dialog != null) {
            if (dialog.isShowing()) {
                dialog.dismiss();
            }
        }
    }

    public void initToolBar(Toolbar toolBar, TextView title, String titleName) {
        if (toolBar != null) {
            RxAppCompatActivity rxAppCompatActivity = (RxAppCompatActivity) getActivity();
            if (rxAppCompatActivity != null) {
                rxAppCompatActivity.setSupportActionBar(toolBar);
                ActionBar actionBar = rxAppCompatActivity.getSupportActionBar();
                if (actionBar != null) {
                    actionBar.setDisplayShowTitleEnabled(false);
                    actionBar.setDisplayHomeAsUpEnabled(false);
                }
                title.setText(titleName == null ? "" : titleName);
            }
        }
    }

    /**
     * 检查权限
     */
    public void checkPermission(OnPermissionListener listener, @NonNull String... permissions) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (permissions.length > 0) {
                this.listener = listener;
                boolean isSuccess = true;
                for (String s : permissions) {
                    if (requireContext().checkSelfPermission(s) != PackageManager.PERMISSION_GRANTED) {
                        requestPermissions(permissions, PERMISSION_CODE);
                        isSuccess = false;
                    }
                }
                if (isSuccess) {
                    listener.onPermissionResult(true);
                }
            }
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == PERMISSION_CODE) {
            if (grantResults.length > 0) {
                boolean isSuccess = true;
                for (int i : grantResults) {
                    if (i != PackageManager.PERMISSION_GRANTED) {
                        isSuccess = false;
                        break;
                    }
                }
                listener.onPermissionResult(isSuccess);
            }
        }
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        if (mPresenter != null) {
            mPresenter.onDetachView();
        }
        if (dialog != null) {
            dialog.dismiss();
            dialog = null;
        }
        if (rootView != null) {
            rootView = null;
        }
        unbinder.unbind();
    }

    public interface OnPermissionListener {
        void onPermissionResult(boolean isSuccess);
    }

}
