package com.tofly.latte_core.net.base;

import com.tofly.latte_core.config.ConfigKeys;
import com.tofly.latte_core.config.Latte;
import com.tofly.latte_core.net.interceptor.LogInterceptor;
import com.tofly.latte_core.net.interceptor.RequestInterceptor;
import com.tofly.latte_core.net.interceptor.ResponseInterceptor;

import java.util.LinkedHashMap;
import java.util.concurrent.TimeUnit;

import io.reactivex.annotations.NonNull;
import okhttp3.OkHttpClient;
import retrofit2.Retrofit;
import retrofit2.adapter.rxjava2.RxJava2CallAdapterFactory;
import retrofit2.converter.gson.GsonConverterFactory;

/**
 * retrofit的封装 基础类
 *
 * @author ychk
 */
public abstract class NetWorkApi {

    /**
     * 设置默认超时时间
     */
    private static final int DEFAULT_TIME = 20;

    /**
     * 是否需要log拦截器 上线时关闭log拦截器
     */
    private boolean isNeedLogInterceptor = Latte.getConfiguration(ConfigKeys.INTERCEPTOR.name());
    /**
     * 用于存储唯一实例
     */
    private final LinkedHashMap<String, Object> linkedHashMap = new LinkedHashMap<>();
    /**
     * baseUrl 域名信息
     */
    private String baseUrl;

    protected NetWorkApi(String baseUrl) {
        this.baseUrl = baseUrl;
    }

    /**
     * 防止多次创建实例，判断一下是否保存有，如果有就直接获取，没有就保存
     */
    public <T> T getRetrofitApi(Class<T> api) {
        T t;
        if (linkedHashMap.get(baseUrl + api.getName()) != null) {
            t = (T) linkedHashMap.get(baseUrl + api.getName());
        } else {
            t = getRetrofit().create(api);
            linkedHashMap.put(baseUrl + api.getName(), t);
        }
        return t;
    }

    /**
     * 初始化Retrofit
     */
    private Retrofit getRetrofit() {
        return new Retrofit.Builder()
                .client(initOkHttp())
                .baseUrl(baseUrl)
                //添加rxjava支持
                .addCallAdapterFactory(RxJava2CallAdapterFactory.create())
                //添加gson解析
                .addConverterFactory(GsonConverterFactory.create())
                .build();
    }

    /**
     * 初始化okHttp
     */
    @SuppressWarnings("UnnecessaryLocalVariable")
    @NonNull
    private OkHttpClient initOkHttp() {
        OkHttpClient.Builder builder = new OkHttpClient().newBuilder();
        //设置读取超时时间
        builder.readTimeout(DEFAULT_TIME, TimeUnit.SECONDS);
        //设置请求超时时间
        builder.connectTimeout(DEFAULT_TIME, TimeUnit.SECONDS);
        //设置写入超时时间
        builder.writeTimeout(DEFAULT_TIME, TimeUnit.SECONDS);
        //设置出现错误进行重新连接。
        builder.retryOnConnectionFailure(true);
        builder.addInterceptor(new RequestInterceptor());
        builder.addInterceptor(new ResponseInterceptor());
        if (isNeedLogInterceptor) {
            builder.addInterceptor(new LogInterceptor());
        }
        OkHttpClient client = builder.build();
        return client;
    }

}
